/*
    Bantumi
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <stdlib.h>
#include "resource.h"
#include "tcpip.h"
#include <SDL_syswm.h>
#include "gui.h"


HWND getWindow() {
	SDL_SysWMinfo info;
	SDL_VERSION(&info.version);
	if (SDL_GetWMInfo(&info) > 0) {
		return info.window;
	} else {
		return GetForegroundWindow();
	}
}

static HWND sdlWindow;
static HHOOK hook;
LRESULT CALLBACK windowHook(int nCode, WPARAM wParam, LPARAM lParam) {
	if (nCode == HC_ACTION) {
		CWPSTRUCT *ptr = (CWPSTRUCT*) lParam;
		if (ptr->hwnd == sdlWindow) {
			if (ptr->message == WM_PAINT || ptr->message == WM_ERASEBKGND) {
				redraw();
			}
/*			if (ptr->message == WM_PAINT)
				printf("WM_PAINT\n");
			if (ptr->message == WM_NCPAINT)
				printf("WM_NCPAINT\n");
			if (ptr->message == WM_SYNCPAINT)
				printf("WM_SYNCPAINT\n");
			if (ptr->message == WM_ERASEBKGND)
				printf("WM_ERASEBKGND\n");
*/
		}

/*		if (ptr->message == WM_SETFOCUS) {
			redraw();
		}
*/
	}
	return CallNextHookEx(0, nCode, wParam, lParam);
}

void initRedrawHook() {
	sdlWindow = getWindow();
	hook = SetWindowsHookEx(WH_CALLWNDPROC, windowHook, NULL, GetCurrentThreadId());
}

void removeRedrawHook() {
	UnhookWindowsHookEx(hook);
}

void flushMessageQueue() {
	MSG msg;
	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
}

void showWarning(const char *str) {

	initRedrawHook();
	MessageBox(getWindow(), str, "Communication error", MB_OK | MB_ICONWARNING);
	removeRedrawHook();
}

static char buffer1[100], buffer2[100];
static const char *lastPort = NULL;
static const char *lastHost = NULL;
static bool client = false;

BOOL CALLBACK TCPDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {
	case WM_INITDIALOG:
		if (lastPort == NULL) {
			sprintf(buffer1, "%d", DEFAULT_TCP_PORT);
			Edit_SetText(GetDlgItem(hDlg, DIALOG_PORT), buffer1);
		} else
			Edit_SetText(GetDlgItem(hDlg, DIALOG_PORT), lastPort);
		if (client && lastHost)
			Edit_SetText(GetDlgItem(hDlg, DIALOG_HOST), lastHost);
//		ghDialog = hDlg;
		return TRUE;
	case WM_SYSCOMMAND:
		if (wParam == SC_CLOSE) {
			EndDialog(hDlg, 0);
			return TRUE;
		}
	case WM_COMMAND:
		if (LOWORD(wParam) == DIALOG_CANCEL) {
			EndDialog(hDlg, 0);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_CONNECT_BUTTON) {
			Edit_GetText(GetDlgItem(hDlg, DIALOG_PORT), buffer1, sizeof(buffer1));
			lastPort = buffer1;
			if (client) {
				Edit_GetText(GetDlgItem(hDlg, DIALOG_HOST), buffer2, sizeof(buffer2));
				lastHost = buffer2;
			}
			EndDialog(hDlg, 1);
			return TRUE;
		}
		break;
	case WM_DESTROY:
//		ghDialog = NULL;
		break;
	}
	return FALSE;
}

Connection *showConnectDialog() {
	client = true;
	initRedrawHook();
	int retval = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(DIALOG_CONNECT), getWindow(), TCPDialogProc, 0);
	removeRedrawHook();
	if (retval) {
		const char *err;
		int port = atoi(lastPort);
		TCPConnection *conn = TCPConnection::startConnect(lastHost, port, &err);
		if (conn == NULL)
			showWarning(err);
		return conn;
	}
	return NULL;
}

Connection *showAcceptDialog() {
	client = false;
	initRedrawHook();
	int retval = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(DIALOG_ACCEPT), getWindow(), TCPDialogProc, 0);
	removeRedrawHook();
	if (retval) {
		const char *err;
		int port = atoi(lastPort);
		TCPConnection *conn = TCPConnection::startAccept(port, &err);
		if (conn == NULL)
			showWarning(err);
		return conn;
	}
	return NULL;
}

BOOL CALLBACK typeDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {
	case WM_INITDIALOG:
		SendMessage(hDlg, DM_SETDEFID, (client ? DIALOG_JOIN_BUTTON : DIALOG_HOST_BUTTON), 0);
		SendMessage(hDlg, WM_NEXTDLGCTL, (WPARAM)GetDlgItem(hDlg, (client ? DIALOG_JOIN_BUTTON : DIALOG_HOST_BUTTON)), TRUE);
		return FALSE; // to indicate that default keyboard focus shouldn't be set
	case WM_SYSCOMMAND:
		if (wParam == SC_CLOSE) {
			EndDialog(hDlg, 0);
			return TRUE;
		}
	case WM_COMMAND:
		if (LOWORD(wParam) == DIALOG_CANCEL) {
			EndDialog(hDlg, 0);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_HOST_BUTTON) {
			EndDialog(hDlg, DIALOG_HOST_BUTTON);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_JOIN_BUTTON) {
			EndDialog(hDlg, DIALOG_JOIN_BUTTON);
			return TRUE;
		}
		break;
	case WM_DESTROY:
		break;
	}
	return FALSE;
}

Connection *showMultiplayerDialog() {
	initRedrawHook();
	int retval = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(DIALOG_TYPE), getWindow(), typeDialogProc, 0);
	removeRedrawHook();
	if (retval != 0) {
		if (retval == DIALOG_HOST_BUTTON)
			return showAcceptDialog();
		else
			return showConnectDialog();
	}
	return NULL;
}

void initGui() {
}

